from typing import Callable, List, Optional, Set

from nltk import word_tokenize  # type: ignore
from nltk.corpus import words  # type: ignore

from elemeta.nlp.extractors.low_level.tokens_count import TokensCount


class OutOfVocabularyCount(TokensCount):
    """
    For a given vocabulary (the default is English vocabulary taken from nltk)
    return the number of words outside of the vocabulary
    """

    def __init__(
        self,
        tokenizer: Callable[[str], List[str]] = word_tokenize,
        vocabulary: Optional[Set[str]] = None,
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        tokenizer: Callable[[str],List[str]]
            a function that splits a text into components
        vocabulary: Optional[Set[str]]
            set of words defined as known words
        """
        super().__init__(
            name=name,
            tokenizer=tokenizer,
            exclude_tokens_list=vocabulary or set(words.words()),
        )

    def extract(self, text: str) -> int:
        return super().extract(text=text.lower())



from typing import Callable, List, Optional

from nltk import word_tokenize  # type: ignore

from elemeta.nlp import english_stopwords
from elemeta.nlp.extractors.low_level.tokens_count import TokensCount


class StopWordsCount(TokensCount):
    """
    Counts the number of stop words
    """

    def __init__(
        self,
        tokenizer: Callable[[str], List[str]] = word_tokenize,
        name: Optional[str] = None,
    ):
        super().__init__(
            name=name, tokenizer=tokenizer, include_tokens_list=english_stopwords
        )



from typing import Optional

from elemeta.nlp.extractors.low_level.abstract_metadata_extractor import (
    AbstractMetadataExtractor,
)


class CapitalLettersRatio(AbstractMetadataExtractor):
    """
    Counts the ratio of capital letters to all letters
    """

    def __init__(self, name: Optional[str] = None):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        """
        super().__init__(name)

    def extract(self, text: str) -> float:
        """case ratio calculator
        returns the ratio of capital letters / length

        Parameters
        ----------
        text: str
            the text to check the ratio on

        Returns
        -------
        float
            the ratio of capital letters / lower letters

        """
        alph = list(filter(str.isalpha, text))
        if len(alph) == 0:
            return 0
        return sum(map(str.isupper, alph)) / len(alph)



from typing import Callable, List, Optional, Set

from nltk import word_tokenize  # type: ignore

from elemeta.nlp.extractors.low_level.must_appear_tokens_parentage import (
    MustAppearTokensPercentage,
)


class MustAppearWordsPercentage(MustAppearTokensPercentage):
    """
    For a given set of words, return the percentage of words that appeared in the text
    """

    def __init__(
        self,
        must_appear: Set[str],
        tokenizer: Callable[[str], List[str]] = word_tokenize,
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        tokenizer: Callable[[str],List[str]]
            a function that splits a text into components
        must_appear: Optional[Set[str]]
            set of words that must appear
        """
        super().__init__(name=name, tokenizer=tokenizer, must_appear=must_appear)



from typing import Callable, Dict, List, Optional, Set

from elemeta.nlp.extractors.low_level.abstract_metadata_extractor import (
    AbstractMetadataExtractor,
)


class UniqueTokenCount(AbstractMetadataExtractor):
    """
    Implementation of AbstractMetadataExtractor class that return the number of unique tokens
     in the text
    """

    def __init__(
        self,
        tokenizer: Callable[[str], List[str]],
        exclude_tokens_list: Optional[Set[str]] = None,
        include_tokens_list: Optional[Set[str]] = None,
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        tokenizer: Callable[[str],List[str]]
            a function that splits a text into components
        exclude_tokens_list: Optional[Set[str]]
            an optional parameter that define if there is tokens that should be excluded
             from the counting
        include_tokens_list: Optional[Set[str]]
            an optional parameter that define a closed set of tokens that we want to count

        """
        super().__init__(name)
        self.tokenizer = tokenizer
        self.exclude_tokens_list = exclude_tokens_list
        self.include_tokens_list = include_tokens_list

    def extract(self, text: str) -> int:
        """counts the number tokens in the text

        Parameters
        ----------
        text: str
            the text to check appearance on

        Returns
        -------
        int
            the number of appearance of a must-appear word list
        """

        tokens: List[str] = self.tokenizer(text)
        corpus: List[str] = []
        if self.exclude_tokens_list:
            corpus = list(
                filter(lambda x: x not in self.exclude_tokens_list, tokens)  # type: ignore
            )
        elif self.include_tokens_list:
            corpus = list(
                filter(lambda x: x in self.include_tokens_list, tokens)  # type: ignore
            )
        else:
            corpus = tokens

        counts: Dict[str, int] = {}
        for token in corpus:
            counts[token] = counts.get(token, 0) + 1
        unique_tokes = filter(lambda k: counts[k] == 1, counts)
        return len(list(unique_tokes))



from typing import Callable, Optional

import textstat  # type: ignore

from elemeta.nlp.extractors.low_level.abstract_metadata_extractor import (
    AbstractMetadataExtractor,
)


class TextComplexity(AbstractMetadataExtractor):
    """
    Return the Flesch Reading Ease Score of the text
    """

    def __init__(
        self,
        metric: Callable[[str], float] = textstat.textstat.flesch_reading_ease,
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        metric: Callable[[str],List[str]]
            a metadata that quantifies the complexity of the text.
            Default is 'flesch reading ease' by the 'textstat' package
        """
        super().__init__(name)
        self.metric = metric

    def extract(self, text: str) -> float:
        return self.metric(text)



from typing import Callable, List, Optional, Set

from elemeta.nlp.extractors.low_level.abstract_metadata_extractor import (
    AbstractMetadataExtractor,
)


class UniqueTokensRatio(AbstractMetadataExtractor):
    """Implementation of AbstractMetadataExtractor class that return the ratio between the
    number of unique tokens to all tokens"""

    def __init__(
        self,
        tokenizer: Callable[[str], List[str]],
        exceptions: Set[str],
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        tokenizer: Callable[[str],List[str]]
            a function that splits a text into components
        exceptions: Set[str]
            tokens to exclude
        """
        super().__init__(name)
        self.tokenizer = tokenizer
        self.exceptions = exceptions

    def extract(self, text: str) -> float:
        """Unique words in text function

        returns the ratio between set(tokens)/len(tokens)
        filters on tokens that are defined as relevant


        Parameters
        ----------
        text: str
            the text we want to find unique words ratio on

        Returns
        -------
        sentiment: float
            the ratio between len(set(tokens that appear once ))/len(set(tokens))

        """
        tokens = self.tokenizer(text)
        corpus = list(filter(lambda x: x not in self.exceptions, tokens))
        counts = {token: 0 for token in corpus}
        for token in corpus:
            counts[token] += 1
        unique_tokes = list(filter(lambda k: counts[k] == 1, counts))
        if len(counts.keys()) == 0:
            return 0
        return len(unique_tokes) / len(counts.keys())



from typing import Optional

from langdetect import DetectorFactory, LangDetectException, detect  # type: ignore

from elemeta.nlp.extractors.low_level.abstract_metadata_extractor import (
    AbstractMetadataExtractor,
)


class DetectLangauge(AbstractMetadataExtractor):
    """
    Returns the language of the text
    """

    def __init__(self, name: Optional[str] = None):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        """
        super().__init__(name)
        DetectorFactory.seed = 42

    def extract(self, text: str) -> str:
        """language detection function

        Parameters
        ----------
        text: str
            the text to detect the language on

        Returns
        -------
        str
           the most likely language of the text
        """
        try:
            return detect(text)
        except LangDetectException:
            return "unknown"



from typing import Callable, List, Optional

import nltk  # type: ignore

from elemeta.nlp.extractors.low_level.avg_token_length import AvgTokenLength


class SentenceAvgLength(AvgTokenLength):
    """
    Gives the average length of sentences in the text
    """

    def __init__(
        self,
        tokenizer: Callable[[str], List[str]] = nltk.sent_tokenize,
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        tokenizer: Callable[[str],List[str]]
            a function that splits a text into components
        """
        super().__init__(name=name, tokenizer=tokenizer)



import re
from typing import Optional

from elemeta.nlp.extractors.low_level.abstract_metadata_extractor import (
    AbstractMetadataExtractor,
)


class RegexMatchCount(AbstractMetadataExtractor):
    """
    For a given regex return the number of matches it has in the text
    """

    def __init__(self, regex: str = ".+", name: Optional[str] = None):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        """
        super().__init__(name)
        self.regex = regex

    def extract(self, text: str) -> int:
        """regex count function

        Parameters
        ----------
        text:str
            a text to run the regex on

        Returns
        -------
        int
           how many times the regex is found in the string

        """
        return len(re.findall(self.regex, text))



from typing import Callable, List, Optional

from better_profanity import profanity  # type: ignore

from elemeta.nlp.extractors import length_check_basic
from elemeta.nlp.extractors.low_level.abstract_metadata_extractor import (
    AbstractMetadataExtractor,
)


class HintedProfanityTokensCount(AbstractMetadataExtractor):
    """Implementation of AbstractMetadataExtractor class that count the number profanity words"""

    def __init__(
        self, tokenizer: Callable[[str], List[str]], name: Optional[str] = None
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        tokenizer: Callable[[str],List[str]]
        a function that splits a text into components.
        """
        super().__init__(name)
        self.tokenizer = tokenizer

    def extract(self, text: str) -> int:
        """
        return the number of profanity words in the text

        Parameters
        ----------
        text: str
            the string to run on
        Returns
        -------
        int
            the number of profanity words in the text
        """
        return length_check_basic(
            self.tokenizer, lambda token: profanity.contains_profanity(token)
        )(text)



from typing import Callable, List


def length_check_basic(
    tokenizer: Callable[[str], List[str]], condition: Callable[[str], bool]
) -> Callable[[str], int]:
    """generic count function generator

    Parameters
    ----------
    tokenizer: Callable[[str],List[str]]
        a function that splits a text into components. Usually into words
    condition: Callable[[str],bool]
        a function that returns true if the token be counted

    Returns
    -------
    Callable[[str],float]
        a function that receives text as string,
        and outputs the number of tokens that are valid according to `condition`.

    """

    def length_check_basic_function(text: str) -> int:
        """length calculator
        calculates the count of a conditioned tokens

        Parameters
        ----------
        text: str
            the text to count on

        Returns
        -------
        float
            the count of tokenized and filtered text

        """
        tokens = tokenizer(text)
        corpus_filter = filter(condition, tokens)
        corpus_filter_count = len(list(corpus_filter))
        return corpus_filter_count

    return length_check_basic_function


def avg_check_basic(
    tokenizer: Callable[[str], List[str]], condition: Callable[[str], bool]
) -> Callable[[str], float]:
    """generic avg counter generator

    Parameters
    ----------
    tokenizer: Callable[[str],List[str]]
        a function that splits a text into components. Usually into words
    condition: Callable[[str],bool]
        a function that returns true if the token be counted

    Returns
    -------
    Callable[[str],float]
        a function that receives text as string,
        and outputs the avg length of tokens that are valid according to `condition`.

    """

    def avg_check_basic_function(text: str) -> float:
        """avg calculator
        calculates the avg length of a conditioned token

        Parameters
        ----------
        text: str
            the text to calculate the avg on

        Returns
        -------
        float
            the average length of a pre-conditioned text

        """
        tokens = tokenizer(text)
        corpus_filter = list(map(len, filter(condition, tokens)))
        corpus_filter_count = (
            sum(corpus_filter) / len(corpus_filter) if len(corpus_filter) != 0 else 0
        )
        return corpus_filter_count

    return avg_check_basic_function



import re
from typing import Callable, List, Optional

from elemeta.nlp.extractors import length_check_basic
from elemeta.nlp.extractors.low_level.abstract_metadata_extractor import (
    AbstractMetadataExtractor,
)


class TokenRegexMatchesCount(AbstractMetadataExtractor):
    """Implementation of AbstractMetadataExtractor class that return number of tokens
    that match the given regex"""

    def __init__(
        self,
        tokenizer: Callable[[str], List[str]],
        regex: str = ".*",
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        tokenizer: Callable[[str],List[str]]
            a function that splits a text into components
        regex: str
            regex to try to match
        """

        super().__init__(name)
        self.tokenizer = tokenizer
        self.regex = regex

    def validator(self, token: str) -> bool:
        """regex check validator
        checks if the token abides by the regex

        Parameters
        ----------
        token: str
            the token check if abides by the regex

        Returns
        -------
        bool
            true if the token abides and false otherwise

        """
        return bool(re.fullmatch(self.regex, token))

    def extract(self, text: str) -> int:
        """
        return the number of matches of the given regex in the text

        Parameters
        ----------
        text: str
            the string to run on
        Returns
        -------
        int
            the number of the given text in the text
        """
        return length_check_basic(self.tokenizer, self.validator)(text)



from typing import Callable, List, Optional, Set

from nltk import word_tokenize  # type: ignore

from elemeta.nlp import english_punctuations
from elemeta.nlp.extractors.low_level.unique_token_count import UniqueTokenCount


class UniqueWordCount(UniqueTokenCount):
    """Implementation of AbstractMetadataExtractor class that return the ratio between the
    number of unique words to all words"""

    def __init__(
        self,
        tokenizer: Callable[[str], List[str]] = word_tokenize,
        exceptions: Set[str] = english_punctuations,
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        tokenizer: Callable[[str],List[str]]
            a function that splits a text into components
        exceptions: Set[str]
            words to exclude
        """
        super().__init__(name=name, tokenizer=tokenizer, exclude_tokens_list=exceptions)



from typing import Callable, List, Optional

from nltk import word_tokenize  # type: ignore

from elemeta.nlp.extractors.low_level.regex_token_matches_count import (
    TokenRegexMatchesCount,
)


class WordRegexMatchesCount(TokenRegexMatchesCount):
    """
    For a given regex return the number of words matching the regex
    """

    def __init__(
        self,
        tokenizer: Callable[[str], List[str]] = word_tokenize,
        regex: str = ".*",
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        tokenizer: Callable[[str],List[str]]
            a function that splits a text into components
        regex: str
            regex to try to match
        """

        super().__init__(name=name, tokenizer=tokenizer, regex=regex)



from typing import Optional

from elemeta.nlp.extractors.high_level.regex_match_count import RegexMatchCount


class LinkCount(RegexMatchCount):
    """
    Counts the number of links in the text
    """

    def __init__(self, name: Optional[str] = None):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        """
        super().__init__(
            regex="^https?:\\/\\/(?:www\\.)?[-a-zA-Z0-9@:%._\\+~#=]{1,256}\\.[a-zA-Z0-9()]{1,6}\\b(?:[-a-zA-Z0-9()@:%_\\+.~#?&\\/=]*)$",  # noqa
            name=name,
        )



from typing import Callable, List, Optional, Set

from elemeta.nlp.extractors.low_level.abstract_metadata_extractor import (
    AbstractMetadataExtractor,
)


class MustAppearTokensPercentage(AbstractMetadataExtractor):
    """Implementation of AbstractMetadataExtractor class that return the ration between
    the number of appearances of tokens from

    given tokens list in the text to all the tokens"""

    def __init__(
        self,
        tokenizer: Callable[[str], List[str]],
        must_appear: Set[str],
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        tokenizer: Callable[[str],List[str]]
            a function that splits a text into components
        must_appear: Optional[Set[str]]
            set of tokes that must appear
        """
        super().__init__(name)
        self.tokenizer = tokenizer
        self.must_appear = must_appear

    def extract(self, text: str) -> float:
        """gives the percentage of the tokens in must_appear set that appeared in the text

        Parameters
        ----------
        text: str
            the text to check appearance on

        Returns
        -------
        float
            the ratio between the number of must-appear tokens to all words
        """
        tokens = self.tokenizer(text)
        corpus = [token for token in tokens if token in self.must_appear]
        if len(self.must_appear) == 0:
            return 0
        return len(set(corpus)) / len(self.must_appear)



from typing import Callable, List, Optional, Set

from nltk import word_tokenize  # type: ignore

from elemeta.nlp import extended_punctuations
from elemeta.nlp.extractors.low_level.tokens_count import TokensCount


class WordCount(TokensCount):
    """
    Gives the number of words in the text
    """

    def __init__(
        self,
        tokenizer: Callable[[str], List[str]] = word_tokenize,
        exclude_tokens_list: Set[str] = extended_punctuations,
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        tokenizer: Callable[[str],List[str]]
            a function that splits a text into components. Usually into words

        exclude_tokens_list: Set[str]
            set of words to exclude
        """
        super().__init__(
            name=name, tokenizer=tokenizer, exclude_tokens_list=exclude_tokens_list
        )



from typing import Optional

from vaderSentiment.vaderSentiment import SentimentIntensityAnalyzer  # type: ignore

from elemeta.nlp.extractors.low_level.abstract_metadata_extractor import (
    AbstractMetadataExtractor,
)


class SentimentPolarity(AbstractMetadataExtractor):
    """
    Returns the Sentiment Polarity
    """

    def __init__(self, name: Optional[str] = None):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        """
        super().__init__(name)

    def extract(self, text: str) -> float:
        """sentiment analysis prediction function

        Parameters
        ----------
        text: str
            the text we want sentiment analysis to run on

        Returns
        -------
        sentiment: float
            between 0 and 1 representing the sentiment.
            0 negative, 1 positive

        """
        sid_obj = SentimentIntensityAnalyzer()
        sentiment_dict = sid_obj.polarity_scores(text)
        sentiment = sentiment_dict["compound"]
        return sentiment



from typing import Callable, List, Optional, Set

from elemeta.nlp.extractors import avg_check_basic
from elemeta.nlp.extractors.low_level.abstract_metadata_extractor import (
    AbstractMetadataExtractor,
)


class AvgTokenLength(AbstractMetadataExtractor):
    """Implementation of AbstractMetadataExtractor class that return the average token length"""

    def __init__(
        self,
        tokenizer: Callable[[str], List[str]],
        tokens_to_exclude: Optional[Set[str]] = None,
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        tokenizer: Callable[[str],List[str]]
            a function that splits a text into components
        tokens_to_exclude: Set[str]
           set of tokens to exclude when computing the metric
        """
        super().__init__(name)
        self.tokenizer = tokenizer
        self.tokens_to_exclude = tokens_to_exclude

    def extract(self, text: str) -> float:
        """
        return the number of average token length in the text

        Parameters
        ----------
        text: str
            the string to run on
        Returns
        -------
        int
            the number of average tokens length in the text
        """
        if self.tokens_to_exclude is None:
            return avg_check_basic(self.tokenizer, lambda _: True)(text)
        else:
            return avg_check_basic(
                self.tokenizer, lambda token: token not in self.tokens_to_exclude  # type: ignore
            )(text)



import os

import pandas

ASSETS_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), "assets")


def get_avengers_endgame_tweets() -> pandas.DataFrame:
    return pandas.read_csv(os.path.join(ASSETS_PATH, "tweets.csv"), encoding="cp1252")


def get_imdb_reviews() -> pandas.DataFrame:
    return pandas.read_csv(os.path.join(ASSETS_PATH, "imdb_dataset.csv"))



from elemeta.nlp.extractors.low_level.abstract_metadata_extractor import (
    AbstractMetadataExtractor,
)


class TextLength(AbstractMetadataExtractor):
    """
    Gives the length of the text
    """

    def extract(self, text: str) -> int:
        """
        text length counter
        returns the length of the text

        Parameters
        ----------
        text: str
            the text to check length on

        Returns
        -------
        int
            the length of the text

        """
        return len(text)



from typing import Callable, List, Optional, Set

from nltk import word_tokenize  # type: ignore

from elemeta.nlp import extended_punctuations
from elemeta.nlp.extractors import length_check_basic
from elemeta.nlp.extractors.low_level.abstract_metadata_extractor import (
    AbstractMetadataExtractor,
)


class PunctuationCount(AbstractMetadataExtractor):
    """
    Counts the number of punctuation marks in the text
    """

    def __init__(
        self,
        tokenizer: Callable[[str], List[str]] = word_tokenize,
        punctuations: Set[str] = extended_punctuations,
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        tokenizer: Callable[[str],List[str]]
            a function that splits a text into components. Usually into words
        punctuations: Set()
            set of punctuations

        """

        super().__init__(name)
        self.tokenizer = tokenizer
        self.punctuations = punctuations

    def extract(self, text: str) -> int:
        """
        return the number of punctuations in the text

        Parameters
        ----------
        text: str
            the string to run on
        Returns
        -------
        int
            the number of punctuations in the text
        """
        return length_check_basic(
            self.tokenizer, lambda token: token in self.punctuations
        )(text)



from typing import Callable, List, Optional

from elemeta.nlp.extractors.low_level.hinted_profanity_token_count import (
    HintedProfanityTokensCount,
)


class HintedProfanityWordsCount(HintedProfanityTokensCount):
    """
    Counts the number of profanity words
    """

    def __init__(
        self,
        tokenizer: Callable[[str], List[str]] = lambda text: text.split(" "),
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        tokenizer: Callable[[str],List[str]]
        a function that splits a text into components.
        """
        super().__init__(name=name, tokenizer=tokenizer)



from typing import Callable, List, Optional, Set

from nltk import word_tokenize  # type: ignore

from elemeta.nlp import english_punctuations
from elemeta.nlp.extractors.low_level.unique_token_ratio import UniqueTokensRatio


class UniqueWordRatio(UniqueTokensRatio):
    """
    Gives the ratio between the number of distinct words (total number of different
    values regardless how many times it appears in the dataset) to the number
    of unique words (total number of values that only appear once in the dataset).
    """

    def __init__(
        self,
        tokenizer: Callable[[str], List[str]] = word_tokenize,
        exceptions: Set[str] = english_punctuations,
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        tokenizer: Callable[[str],List[str]]
            a function that splits a text into components
        exceptions: Set[str]
            words to exclude
        """
        super().__init__(name=name, tokenizer=tokenizer, exceptions=exceptions)



import re
from abc import ABC, abstractmethod
from typing import Any, Optional


class AbstractMetadataExtractor(ABC):
    """
    Representation of a MetadataExtractor
    This class holds a function to be run to extract the metadata value and the name
     of the metadata
    """

    def __init__(self, name: Optional[str] = None):
        """initializer for the Metric object

        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        """
        if name:
            self.name = name
        else:
            self.name = re.sub(r"(?<!^)(?=[A-Z])", "_", self.__class__.__name__).lower()

    @abstractmethod
    def extract(self, text: str) -> Any:
        """
        This function will extract the metric from the text
        Parameters
        ----------
        text: str

        Returns
        -------
        Any
            the metadata extracted from text
        """
        raise NotImplementedError

    def __call__(self, text: str):
        """
        run self.extract on the given text

        Parameters
        ----------
        text: str

        Returns
        -------
        Any
            the metadata extracted from text
        """
        return self.extract(text)



from typing import Callable, List, Optional, Set

from elemeta.nlp.extractors import length_check_basic
from elemeta.nlp.extractors.low_level.abstract_metadata_extractor import (
    AbstractMetadataExtractor,
)


class TokensCount(AbstractMetadataExtractor):
    """Implementation of AbstractMetadataExtractor class that return the number of sentences
    in the text"""

    def __init__(
        self,
        tokenizer: Callable[[str], List[str]],
        exclude_tokens_list: Optional[Set[str]] = None,
        include_tokens_list: Optional[Set[str]] = None,
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        tokenizer: Callable[[str],List[str]]
            a function that splits a text into components
        exclude_tokens_list: Optional[Set[str]]
            an optional parameter that define if there is tokens that should be excluded
             from the counting
        include_tokens_list: Optional[Set[str]]
            an optional parameter that define a closed set of tokens that we want to count

        """
        super().__init__(name)
        self.tokenizer = tokenizer
        self.exclude_tokens_list = exclude_tokens_list
        self.include_tokens_list = include_tokens_list

    def extract(self, text: str) -> int:
        """counts the number tokens in the text

        Parameters
        ----------
        text: str
            the text to check appearance on

        Returns
        -------
        int
            the number of appearance of a must-appear word list
        """

        if self.exclude_tokens_list:
            return length_check_basic(
                self.tokenizer,
                lambda token: token.lower() not in self.exclude_tokens_list,  # type: ignore
            )(text)
        elif self.include_tokens_list:
            return length_check_basic(
                self.tokenizer,
                lambda token: token.lower() in self.include_tokens_list,  # type: ignore
            )(text)
        else:
            return length_check_basic(self.tokenizer, lambda _: True)(text)



from typing import Any, Dict, List, Optional

from pandas import DataFrame

from elemeta.nlp.extractors.high_level.acronym_count import AcronymCount
from elemeta.nlp.extractors.high_level.avg_word_length import AvgWordLength
from elemeta.nlp.extractors.high_level.capital_letters_ratio import CapitalLettersRatio
from elemeta.nlp.extractors.high_level.date_count import DateCount
from elemeta.nlp.extractors.high_level.email_count import EmailCount
from elemeta.nlp.extractors.high_level.emoji_count import EmojiCount
from elemeta.nlp.extractors.high_level.hashtag_count import HashtagCount
from elemeta.nlp.extractors.high_level.hinted_profanity_sentence_count import (
    HintedProfanitySentenceCount,
)
from elemeta.nlp.extractors.high_level.hinted_profanity_words_count import (
    HintedProfanityWordsCount,
)
from elemeta.nlp.extractors.high_level.link_count import LinkCount
from elemeta.nlp.extractors.high_level.mention_count import MentionCount
from elemeta.nlp.extractors.high_level.must_appear_words_percentage import (
    MustAppearWordsPercentage,
)
from elemeta.nlp.extractors.high_level.number_count import NumberCount
from elemeta.nlp.extractors.high_level.out_of_vocabulary_count import (
    OutOfVocabularyCount,
)
from elemeta.nlp.extractors.high_level.punctuation_count import PunctuationCount
from elemeta.nlp.extractors.high_level.regex_match_count import RegexMatchCount
from elemeta.nlp.extractors.high_level.sentence_avg_length import SentenceAvgLength
from elemeta.nlp.extractors.high_level.sentence_count import SentenceCount
from elemeta.nlp.extractors.high_level.sentiment_polarity import SentimentPolarity
from elemeta.nlp.extractors.high_level.sentiment_subjectivity import (
    SentimentSubjectivity,
)
from elemeta.nlp.extractors.high_level.special_chars_count import SpecialCharsCount
from elemeta.nlp.extractors.high_level.stop_words_count import StopWordsCount
from elemeta.nlp.extractors.high_level.syllable_count import SyllableCount
from elemeta.nlp.extractors.high_level.text_complexity import TextComplexity
from elemeta.nlp.extractors.high_level.text_length import TextLength
from elemeta.nlp.extractors.high_level.unique_word_count import UniqueWordCount
from elemeta.nlp.extractors.high_level.unique_word_ratio import UniqueWordRatio
from elemeta.nlp.extractors.high_level.word_count import WordCount
from elemeta.nlp.extractors.high_level.word_regex_matches_count import (
    WordRegexMatchesCount,
)
from elemeta.nlp.extractors.low_level.abstract_metadata_extractor import (
    AbstractMetadataExtractor,
)

compute_intensive_metrics = [
    SentimentPolarity(),
    SentimentSubjectivity(),
    HintedProfanityWordsCount(),
    HintedProfanitySentenceCount(),
]
metrics = [
    EmojiCount(),
    TextComplexity(),
    UniqueWordRatio(),
    UniqueWordCount(),
    WordRegexMatchesCount(),
    NumberCount(),
    OutOfVocabularyCount(),
    MustAppearWordsPercentage(must_appear=set()),
    SentenceCount(),
    SentenceAvgLength(),
    WordCount(),
    AvgWordLength(),
    TextLength(),
    StopWordsCount(),
    PunctuationCount(),
    SpecialCharsCount(),
    CapitalLettersRatio(),
    RegexMatchCount(),
    EmailCount(),
    LinkCount(),
    HashtagCount(),
    MentionCount(),
    SyllableCount(),
    AcronymCount(),
    DateCount(),
]


class MetadataExtractorsRunner:
    """
    This class used to run multiple MetadataExtractors on a text

    Attributes
    ----------
    metadata_extractors : Optional[List[AbstractMetadataExtractor]]
        a list of `MetadataExtractor`s to run,
        if not supplied will run with all metadata extractors.

    Methods
    -------
    run(text)
        runs all the metadata extractors on the input text
    run_on_dataframe(df,text_column)
        runs all the metadata extractors on the given text_column in the given dataframe
         and return new dataframe with metadata values as columns
    """

    def __init__(
        self,
        metadata_extractors: Optional[List[AbstractMetadataExtractor]] = None,
    ):
        """Representation of a df, text column, and list of `AbstractMetadataExtractor` to run on
        This is a wrapper for a pandas df,
        the column holding the text,
        and the `AbstractMetadataExtractor`s to run on.

        Parameters
        ----------
        metadata_extractors : Optional[List[AbstractMetadataExtractor]]
            a list of `AbstractMetadataExtractor`s to run over. Runs on all of them independently.
            if not supplied will initialize a list of all metrics with the default configuration

        """
        self.metadata_extractors: List[AbstractMetadataExtractor] = (
            metadata_extractors or metrics.copy()
        )

    def run(self, text: str) -> Dict[str, Any]:
        """run metrics on list of text

        Parameters
        ----------
        text: str
            the text to run all metrics on

        Returns
        -------
        metadata_value_dict: Dict[str, Any]
            returns a dictionary of extractor name and the metadata value

        """
        return {
            metric.name: metric.extract(text) for metric in self.metadata_extractors
        }

    def run_on_dataframe(self, dataframe: DataFrame, text_column: str) -> DataFrame:
        """return new dataframe with all metadata extractors values
        Parameters
        ----------
        dataframe: DataFrame
            dataframe with the text column
        text_column: str
            the name of the text column in the given dataframe

        Returns
        -------
        dataframe: DataFrame
            dataframe with the values of the metadata extractors as new columns
        """
        dataframe_to_return = dataframe.copy()
        if text_column not in dataframe_to_return.columns:
            raise AssertionError(
                f"The given text_column:'{text_column}' doesn't exist in the given dataframe"
            )

        names = set()
        for metric in self.metadata_extractors:
            assert (
                metric.name not in names
            ), f"more than one metric have the name {metric.name}"
            names.add(metric.name)

        data_frame_text = dataframe_to_return[text_column]
        for metric in self.metadata_extractors:
            dataframe_to_return.loc[:, metric.name] = data_frame_text.map(
                metric.extract
            )

        return dataframe_to_return

    def add_metadata_extractor(
        self, metadata_extractor: AbstractMetadataExtractor
    ) -> None:
        self.metadata_extractors.append(metadata_extractor)



from typing import Optional

from elemeta.nlp.extractors.high_level.regex_match_count import RegexMatchCount


class MentionCount(RegexMatchCount):
    """
    Counts the number of mentions (word in the format @someones_name)
    """

    def __init__(self, name: Optional[str] = None):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        """
        super().__init__(regex="(^|[^@\w])@(\w+)", name=name)  # noqa: W605



from typing import Optional

import emoji

from elemeta.nlp.extractors.low_level.abstract_metadata_extractor import (
    AbstractMetadataExtractor,
)


class EmojiCount(AbstractMetadataExtractor):
    """
    Counts the number of emojis in the text
    """

    def __init__(self, name: Optional[str] = None):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        """
        super().__init__(name)

    def extract(self, text: str) -> int:
        """emoji counter function

        Parameters
        ----------
        text: str
            the text to count emoji on

        Returns
        -------
        int
            the number of emojis in the text

        """
        return emoji.emoji_count(text)



r"""
requirements for the nlp pacakge
"""
import string

import nltk  # type: ignore
from nltk.corpus import stopwords  # type: ignore

# download files for common use
nltk.download("punkt", quiet=True)
nltk.download("stopwords", quiet=True)
nltk.download("words", quiet=True)

"""
English stopwords
"""
english_stopwords = set(stopwords.words("english"))

"""
Punctuations from nltk
"""
string_punctuations = set(string.punctuation)

english_punctuations = {
    "...",
    ".",
    '"',
    "'",
    "-",
    "?",
    ",",
    "'",
    ":",
    ";",
    "_",
    "[",
    "]",
    "(",
    ")",
    "{",
    "}",
    "!",
}

"""
Set of all punctuations in english
"""
extended_punctuations = string_punctuations.union(english_punctuations)

"""
Set of all special charters
"""
special_chars = {
    "",
    "!",
    "@",
    "#",
    "$",
    "%",
    "'",
    "^",
    "&",
    "*",
    "<",
    ">",
    "?",
    "/",
    "\\",
    "_",
    "+",
    "-",
    "=",
    "~",
    "`",
    "[",
    "]",
    "{",
    "}",
    ".",
    "",
}



from typing import Callable, List, Optional, Set

from nltk import word_tokenize  # type: ignore

from elemeta.nlp import special_chars
from elemeta.nlp.extractors.low_level.tokens_count import TokensCount


class SpecialCharsCount(TokensCount):
    """
    Counts the number of special characters in the text
    """

    def __init__(
        self,
        tokenizer: Callable[[str], List[str]] = word_tokenize,
        specials: Set[str] = special_chars,
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        tokenizer: Callable[[str],List[str]]
            a function that splits a text into components. Usually into words
        specials: set()
            a set of special characters (,.!@#$...)
        """
        super().__init__(name=name, tokenizer=tokenizer, include_tokens_list=specials)



from typing import Callable, List, Optional, Set

from nltk import word_tokenize  # type: ignore

from elemeta.nlp import extended_punctuations
from elemeta.nlp.extractors.low_level.avg_token_length import AvgTokenLength


class AvgWordLength(AvgTokenLength):
    """
    Gives the average length of the words in the text
    """

    def __init__(
        self,
        tokenizer: Callable[[str], List[str]] = word_tokenize,
        exclude_list: Set[str] = extended_punctuations,
        name: Optional[str] = None,
    ):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        tokenizer: Callable[[str],List[str]]
            a function that splits a text into components
        exclude_list: Set[str]
           set of words to exclude when computing the metric
        """
        super().__init__(name=name, tokenizer=tokenizer, tokens_to_exclude=exclude_list)



from typing import Optional

from textblob import TextBlob  # type: ignore

from elemeta.nlp.extractors.low_level.abstract_metadata_extractor import (
    AbstractMetadataExtractor,
)


class SentimentSubjectivity(AbstractMetadataExtractor):
    """
    Returns the Sentiment Subjectivity
    """

    def __init__(self, name: Optional[str] = None):
        """
        Parameters
        ----------
        name: Optional[str]
            name to of the metadata of not given will extract the name from the class name
        """
        super().__init__(name)

    def extract(self, text: str) -> float:
        """sentiment subjectivity prediction function

        Parameters
        ----------
        text: str
            the text we want sentiment subjectivity to run on

        Returns
        -------
        sentiment: float
            return subjectivity score as a float within the range [0.0, 1.0]
        where 0.0 is very objective and 1.0 is very subjective.
        """
        return TextBlob(text).sentiment.subjectivity
